require 'net/sftp'

require 'pathname'

require "filters/pass"
require "filters/git_ignore"

require "file_diff_map"

module Sftp
  module Sync
    class Client

      attr_reader :config, :session, :filter


      def initialize(_config)
        @config = _config
        @session = Net::SFTP.start(config.host, config.username, config.sftp_opts) 

        # set filter
        @filter = Filters::Pass.new

        if config.filter?
          @filter = Filters::GitIgnore.new(IO.readlines(config.filter))
        end
      end

      def close

        @session.close
      end

      def list

 
  
        # list the entries in a directory
        session.dir.glob("/", "**/*") do |entry|
          puts entry.name unless entry.directory? || filter.match(entry.name)
        end
    

      end


      def diff

        diff_map = build_diff_map


               # list the entries in a directory
        session.dir.glob("/", "**/*") do |entry|

          unless entry.directory? || filter.match(entry.name)
            case diff_map.update(entry.name , entry.attributes.size ) 
            when :diff
              puts "*  #{entry.name}"
            when :match
              puts "   #{entry.name}" if config.verbose?
            when :remote
              puts "+  #{entry.name}"
            end

          end
        end

        diff_map.local_entries.each do |entry|
 
          puts "-  #{entry.name}"
        
        end
      end



      def pull
  
        downloads = []

        diff_map = build_diff_map

        output_path = File.expand_path(config.output_dir)

        # list the entries in a directory
        session.dir.glob("/", "**/*") do |entry|
         
          unless entry.directory? || filter.match(entry.name)

            src = entry.name
            dest = File.join(output_path, entry.name)

            dest_dir = File.dirname(dest)
            FileUtils.mkdir_p dest_dir unless Dir.exists?(dest_dir)

            puts "#{src} => #{dest}"

            downloads << session.download(src , dest)

            ## 
            diff_map.update(entry.name , entry.attributes.size ) 

            

          end


        end

        downloads.each { |d| d.wait }


    
        diff_map.local_entries.each do |entry|

          target = File.join(output_path, entry.name)


          if config.remove_local?
            if File.exist?(target)
              puts "removing  #{target}" 
              File.delete(target)
            end
          else
             puts "#{target} in local but not remote, use --remove_local flag to auto remove" 
          end

        end

        

      end



      private 

      def build_diff_map
        diff_map = FileDiffMap.new
   
        output_pathname = Pathname.new(File.expand_path(config.output_dir))

        ## enumerate local files
        Dir[ File.join(output_pathname.to_s, '**', '*') ].each do |local_entry|



          relative_path = Pathname.new(local_entry).relative_path_from(output_pathname).to_s

          diff_map.add(relative_path, (File.size?(local_entry)|| 0))  unless File.directory?(local_entry) || filter.match(relative_path)
        end

        diff_map
      end




                # # upload a file or directory to the remote host
          # sftp.upload!("/path/to/local", "/path/to/remote")

          # # download a file or directory from the remote host
          # sftp.download!("/path/to/remote", "/path/to/local")

          # # grab data off the remote host directly to a buffer
          # data = sftp.download!("/path/to/remote")

          # # open and write to a pseudo-IO for a remote file
          # sftp.file.open("/path/to/remote", "w") do |f|
          #   f.puts "Hello, world!\n"
          # end

          # # open and read from a pseudo-IO for a remote file
          # sftp.file.open("/path/to/remote", "r") do |f|
          #   puts f.gets
          # end

          # # create a directory
          # sftp.mkdir! "/path/to/directory"

    end
  end
end