require "active_support/hash_with_indifferent_access"

module Sftp
  module Sync
    class Config

      attr_reader :options
      def initialize(_options = {})
        @options = ActiveSupport::HashWithIndifferentAccess.new(_options)


        #required
        options[:host] = ENV["SFTP_SYNC_HOST"] unless options[:host].present?
        raise "missing host. Set SFTP_SYNC_HOST env." unless options[:host].present?

        options[:username] = ENV["SFTP_SYNC_USERNAME"] unless options[:username].present?
        raise "missing username. Set SFTP_SYNC_USERNAME env." unless options[:username].present?


        #optional
        options[:port] = ENV["SFTP_SYNC_PORT"] unless options[:port].present?
        options[:password] = ENV["SFTP_SYNC_PASSWORD"] unless options[:password].present?



      end


      def host
        options[:host]
      end

      def username
        options[:username]
      end

      def password
        options[:password]
      end

      def port
        options[:port]
      end

      def sftp_opts 
        {}.tap do |opts|
          opts[:password] = password unless password.nil?
          opts[:port] = port unless port.nil?
        end
      end

      def filter?
        filter.present?
      end

      def filter
        if options[:filter].present?
          options[:filter]
        elsif File.exist?(".gitignore") && options[:gitignore]
          ".gitignore" 
        else 
          nil
        end
      end

      def output_dir
        options[:output_dir] || ""
      end

      def remove_local?
        options[:remove_local]
      end

      def verbose?
        options[:verbose] || false
      end
    end
  end
end
